#!/bin/bash
############################################################################
# 0DaysDateFix v1.3 by Turranius                                           #
####                                                                    ####
# This little script will change the date on any dated dirs, be it 0days   #
# or Mp3 folders. The only requirement is that the dirs in in the format   #
# "MMDD", like 0628.                                                       #
# It is useful for 2 reasons. 1: It looks much better when listing and     #
# 2: Automatic delete scripts, like Spacemaker, can easely see which dir   #
# is the oldest so it does not delete the wrong ones.                      #
####                                                                    ####
# Installation:                                                            #
# Put this script in /glftpd/bin, or wherever you like really.             #
# Make it executable.                                                      #
# Copy the binary file_date to /glftpd/bin and make that executable too.   #
# If you dont like binaries, the source is included so you can compile it  #
# yourself.                                                                #
# Try running "file_date AnyFile" and it should return the date info on    #
# the folder.                                                              # 
# Now then, edit the settings below:                                       #
# DATEDDIRS can be any number of folders containing dated dirs, space      #
# separated.                                                               #
#                                                                          #
# To rename folders from mmdd to mmddyy, add a ~ after its path.           #
# This is for archives more then a year old.                               #
# It is recomended to rename old archives to MMDDYY if you set this script #
# to keep the dates tidy on them because if the folder is MMDDYY, this     #
# script will use that year to update the time. If not, I will read the    #
# date on the folder. That can change if you modify something in it, thus  #
# the script will timestamp it with the wrong year. Its less likely that   #
# the name of the actual folder change...                                  #
#                                                                          #
# Anyway..                                                                 #
# Any folders contained in these dated dirs can be excluded in EXCLUDE.    #
# Separate the excludes with a |                                           #
# Crontab it or just run it manually as often or seldom as you like.       #
# To run it 15 minutes past midnight, each day, do:                        #
# 15 0 * * * /glftpd/bin/0daysdatefix.sh                                   #
#                                                                          #
# Oh yeah. Set the "filedate" option to whereever you put file_date.       #
#                                                                          # 
# If you wish to see whats happening when its running, start it with the   #
# variable 'display' (0daysdatefix.sh display).                            #
####                                                                    ####
# Changelog:                                                               #
# 1.3   : As Peza pointed out, it fudged up the year on old releases.      #
#         It basically took releases from 2002 and stamped them as 2003.   #
#         I thought that it would leave the year alone on the              #
#         folder if I didnt specify it in the "touch" part. Wrong! It      #
#         took the current year. I am truly sorry if I fucked things up    #
#         for you, You can always retouch wrong folder with:               #
#         touch FolderName -t 200200000000                                 #
#         Thats YYYYMMDDHHMM. MMDDHHMM can be 0's though.  We only need    #
#         the correct year on it, then this script will do the rest.       #
#         Anyway, while I was at it, I removed the REDHAT8 setting and     #
#         cruxis made me a small binary to check the year instead of using #
#         ls --full-time. This should work better as ls gave different     #
#         output for every linux distro out there.                         #
#         So, to upgrade, just copy your paths to the new script and       #
#         put the file_info binary into /glftpd/bin or compile the source. #
#                                                                          #
# 1.2.1 : Added option REDHAT8. I noticed the full time argument being     #
#         different on RH8, so added an option for it. I suspect it is the #
#         same with different distros, ala slackware etc.                  #
#         If it throws out a lot of touch errors or "Cant move folder into #
#         itself", try flipping this option.                               #
#                                                                          #
#         Fixed it so that once it renames a folder, it will still keep    #
#         the time on it up to date, but then it will go by the name of    #
#         the folder instead of trying to check creation date like it does #
#         when renaming from 4 to 6 chars.                                 #
#         Previously, once it renamed a folder to 6 digits, it stopped     #
#         updated the time on it.                                          #
#                                                                          #
#         Added more output when running it with the argument 'display'.   #
#                                                                          #
# 1.2   : Added option to rename dated folders from mmdd to mmddyy.        #
#         Only enable this in archive dirs.                                #
# 1.1.1 : Fixed it where it would not run the first 9 days of the month.   #
# 1.1 : Handles an infinite amount of dated dirs.                          #
#       Excludes are working.                                              #
#       Better (?) documented, hehe.                                       #
# 1.0 : Initial release. Only handles one dir.                             #
############################################################################
#    Contact Turranius on efnet (turranius/turran/turr|away/turr|work).    #
#     http://www.grandis.nu/glftpd/ <-> http://grandis.mine.nu/glftpd/     #
############################################################################

DATEDDIRS="
/glftpd/site/0DAYS
/glftpd/site/Archive/0DAYS/0DAYS1~
"

EXCLUDE='lost\+found|!Today|!Yesterday' # | separated

filedate=/glftpd/bin/file_date


############################################################
## No changes below needed                                 #
############################################################

for dir in $DATEDDIRS; do
  rename="$( echo $dir | grep "~" )"
  if [ "$rename" != "" ]; then
    dir="$( echo $dir | awk -F"~" '{print $1}' )"
    renameit="yes"
  else
    renameit="no"
  fi

  cd $dir

  for i in `ls -f -A | egrep -v $EXCLUDE`; do
    if [ "$1" = "display" ]; then
      echo "Checking: $dir/$i"
    fi
    yearf="$( $filedate $i | awk '{print $5}' )"
    lenght="$( echo $i | wc -L | tr -d ' ' )"
    if [ "$renameit" = "yes" ]; then
      if [ "$lenght" = "4" ]; then
        yearmove="$( echo $yearf | cut -b3-4 )"
        if [ "$1" = "display" ]; then
          echo " mv -f $i $i$yearmove"
        fi
        mv -f $i $i$yearmove
        folder="$i$yearmove"
        if [ "$1" = "display" ]; then
          echo " touch $folder -t $yearf$i"0000""
        fi
        touch $folder -t $yearf$i"0000"
      fi
      if [ "$lenght" = "6" ]; then
        datef="$( echo $i | cut -b1-4 )"
        year="$( echo $i | cut -b5-6 )"
        if [ "$year" -gt "50" ]; then
          yearf="19"$year
        else
          yearf="20"$year
        fi
        if [ "$1" = "display" ]; then
          echo " touch $i -t $yearf$datef"0000""
        fi
        touch $i -t $yearf$datef"0000"
      fi
    else
      if [ "$lenght" = "4" ]; then
        if [ "$1" = "display" ]; then
          echo " touch $i -t $yearf$i"0000""
        fi
        touch $i -t $yearf$i"0000"
      else
        if [ "$1" = "display" ]; then
          echo "$i is not 4 chars. Skipping"
        fi
      fi
    fi
  done
done

exit 0
